==> libtangle_0.0.20100215.tar.gz: libtangle-0.0.20100215/tangle/tangle-grid.c <==
/*
 * tangle-grid.c
 *
 * This file is part of Tangle Toolkit - A graphical actor library based on Clutter Toolkit
 *
 * (c) 2010 Henrik Hedberg <henrik.hedberg@innologies.fi>
 *
 */

#include "tangle-grid.h"

/**
 * SECTION:tangle-grid
 * @Short_description: An actor that displays grid lines
 * @Title: TangleGrid
 */
 
G_DEFINE_TYPE(TangleGrid, tangle_grid, TANGLE_TYPE_ACTOR);

enum {
	PROP_0,
	PROP_GRID_SPACING_X,
	PROP_GRID_SPACING_Y,
	PROP_BACKGROUND_COLOR,
	PROP_LINE_COLOR
};

struct _TangleGridPrivate {
	gfloat grid_spacing_x;
	gfloat grid_spacing_y;
	ClutterColor background_color;
	ClutterColor line_color;
	guint background_color_set : 1;
	guint line_color_set : 1;
};

static ClutterColor black = { 0, 0, 0, 255 };

ClutterActor* tangle_grid_new(gfloat grid_spacing_x, gfloat grid_spacing_y) {

	return CLUTTER_ACTOR(g_object_new(TANGLE_TYPE_GRID, "grid-spacing-x", grid_spacing_x, "grid-spacing-y", grid_spacing_y, NULL));
}

ClutterActor* tangle_grid_new_with_line_color(gfloat grid_spacing_x, gfloat grid_spacing_y, ClutterColor* line_color) {

	return CLUTTER_ACTOR(g_object_new(TANGLE_TYPE_GRID, "grid-spacing-x", grid_spacing_x, "grid-spacing-y", grid_spacing_y, "line-color", line_color, NULL));
}

gfloat tangle_grid_get_grid_spacing_x(TangleGrid* grid) {

	return grid->priv->grid_spacing_x;
}

void tangle_grid_set_grid_spacing_x(TangleGrid* grid, gfloat grid_spacing_x) {
	g_return_if_fail(grid_spacing_x >= 0.0);

	if (grid->priv->grid_spacing_x != grid_spacing_x) {
		grid->priv->grid_spacing_x = grid_spacing_x;
		clutter_actor_queue_redraw(CLUTTER_ACTOR(grid));
		g_object_notify(G_OBJECT(grid), "grid-spacing-x");
	}
}

gfloat tangle_grid_get_grid_spacing_y(TangleGrid* grid) {

	return grid->priv->grid_spacing_y;
}

void tangle_grid_set_grid_spacing_y(TangleGrid* grid, gfloat grid_spacing_y) {
	g_return_if_fail(grid_spacing_y >= 0.0);

	if (grid->priv->grid_spacing_y != grid_spacing_y) {
		grid->priv->grid_spacing_y = grid_spacing_y;
		clutter_actor_queue_redraw(CLUTTER_ACTOR(grid));
		g_object_notify(G_OBJECT(grid), "grid-spacing-y");
	}
}

void tangle_grid_get_background_color(TangleGrid* grid, ClutterColor* color) {
	g_return_if_fail(color != NULL);

	*color = grid->priv->background_color;
}

void tangle_grid_set_background_color(TangleGrid* grid, const ClutterColor* color) {
	if (!clutter_color_equal(&grid->priv->background_color, color)) {
		if (color) {
			grid->priv->background_color = *color;
			grid->priv->background_color_set = TRUE;
		} else {
			grid->priv->background_color_set = FALSE;
		}
		g_object_notify(G_OBJECT(grid), "background-color");
		clutter_actor_queue_redraw(CLUTTER_ACTOR(grid));
	}
}

void tangle_grid_get_line_color(TangleGrid* grid, ClutterColor* color) {
	g_return_if_fail(color != NULL);

	*color = grid->priv->line_color;
}

void tangle_grid_set_line_color(TangleGrid* grid, const ClutterColor* color) {
	if (!clutter_color_equal(&grid->priv->line_color, color)) {
		if (color) {
			grid->priv->line_color = *color;
			grid->priv->line_color_set = TRUE;
		} else {
			grid->priv->line_color_set = FALSE;
		}
		g_object_notify(G_OBJECT(grid), "line-color");
		clutter_actor_queue_redraw(CLUTTER_ACTOR(grid));
	}
}

static void tangle_grid_paint_aligned(TangleActor* actor, gfloat width, gfloat height) {
	TangleGrid* grid;
	gfloat x, y;
	guint8 alpha;
	
	grid = TANGLE_GRID(actor);
	
	if (grid->priv->background_color_set) {
		alpha = clutter_actor_get_paint_opacity(CLUTTER_ACTOR(actor)) * grid->priv->background_color.alpha / 255;
		cogl_set_source_color4ub(grid->priv->background_color.red,
			 		 grid->priv->background_color.green,
			 		 grid->priv->background_color.blue,
					 alpha);
		cogl_rectangle(0.0, 0.0, width, height);
	}

	if (grid->priv->line_color_set) {
		cogl_set_source_color4ub(grid->priv->background_color.red,
			 	 grid->priv->background_color.green,
			 	 grid->priv->background_color.blue,
				 alpha);

		if (grid->priv->grid_spacing_x > 0.0) {
			for (x = 0.0; x < width; x += grid->priv->grid_spacing_x) {
				cogl_path_move_to(x, 0.0);
				cogl_path_line_to(x, height);
			}
			cogl_path_stroke();
		}
		if (grid->priv->grid_spacing_y > 0.0) {
			for (y = 0.0; y < height; y += grid->priv->grid_spacing_y) {
				cogl_path_move_to(0.0, y);
				cogl_path_line_to(width, y);
			}
			cogl_path_stroke();
		}
	}
}

static void tangle_grid_set_property(GObject* object, guint prop_id, const GValue* value, GParamSpec* pspec) {
	TangleGrid* grid;
	
	grid = TANGLE_GRID(object);

	switch (prop_id) {
		case PROP_GRID_SPACING_X:
			tangle_grid_set_grid_spacing_x(grid, g_value_get_float(value));
			break;
		case PROP_GRID_SPACING_Y:
			tangle_grid_set_grid_spacing_y(grid, g_value_get_float(value));
			break;
		case PROP_BACKGROUND_COLOR:
			tangle_grid_set_background_color(grid, clutter_value_get_color(value));
			break;
		case PROP_LINE_COLOR:
			tangle_grid_set_line_color(grid, clutter_value_get_color(value));
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
			break;
	}
}

static void tangle_grid_get_property(GObject* object, guint prop_id, GValue* value, GParamSpec* pspec) {
        TangleGrid* grid;

	grid = TANGLE_GRID(object);

        switch (prop_id) {
		case PROP_GRID_SPACING_X:
			g_value_set_float(value, grid->priv->grid_spacing_x);
			break;
		case PROP_GRID_SPACING_Y:
			g_value_set_float(value, grid->priv->grid_spacing_y);
			break;
		case PROP_BACKGROUND_COLOR:
			clutter_value_set_color(value, &grid->priv->background_color);
			break;
		case PROP_LINE_COLOR:
			clutter_value_set_color(value, &grid->priv->line_color);
			break;
	        default:
		        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
		        break;
        }
}

static void tangle_grid_finalize(GObject* object) {
	G_OBJECT_CLASS(tangle_grid_parent_class)->finalize(object);
}

static void tangle_grid_dispose(GObject* object) {
	G_OBJECT_CLASS(tangle_grid_parent_class)->dispose(object);
}

static void tangle_grid_class_init(TangleGridClass* grid_class) {
	GObjectClass* gobject_class = G_OBJECT_CLASS(grid_class);
	TangleActorClass* actor_class = TANGLE_ACTOR_CLASS(grid_class);

	gobject_class->finalize = tangle_grid_finalize;
	gobject_class->dispose = tangle_grid_dispose;
	gobject_class->set_property = tangle_grid_set_property;
	gobject_class->get_property = tangle_grid_get_property;

	actor_class->paint_aligned = tangle_grid_paint_aligned;

	/**
	 * TangleGrid:grid-spacing-x:
	 */
	g_object_class_install_property(gobject_class, PROP_GRID_SPACING_X,
	                                g_param_spec_float("grid-spacing-x",
	                                                   "Grid spacing x",
	                                                   "The horizontal spacing of grid lines",
	                                                   0.0, G_MAXFLOAT, 0.0,
	                                                   G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
	/**
	 * TangleGrid:grid-spacing-y:
	 */
	g_object_class_install_property(gobject_class, PROP_GRID_SPACING_Y,
	                                g_param_spec_float("grid-spacing-y",
	                                                   "Grid spacing y",
	                                                   "The vertical spacing of grid lines",
	                                                   0.0, G_MAXFLOAT, 0.0,
	                                                   G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
	/**
	 * TangleGrid:background-color:
	 */
	g_object_class_install_property(gobject_class, PROP_BACKGROUND_COLOR,
	                                clutter_param_spec_color("background-color",
	                                                	 "Background color",
	                                                	 "The background color of the grid",
	                                                	 NULL,
	                                                	 G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
	/**
	 * TangleGrid:line-color:
	 */
	g_object_class_install_property(gobject_class, PROP_LINE_COLOR,
	                                clutter_param_spec_color("line-color",
	                                                	 "Line color",
	                                                	 "The line stroke color of the grid",
	                                                	 &black,
	                                                	 G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));


	g_type_class_add_private (gobject_class, sizeof (TangleGridPrivate));
}

static void tangle_grid_init(TangleGrid* grid) {
	grid->priv = G_TYPE_INSTANCE_GET_PRIVATE(grid, TANGLE_TYPE_GRID, TangleGridPrivate);
}

==> libtangle_0.0.20100307.tar.gz: libtangle-0.0.20100307/tangle/tangle-grid.c <==
/*
 * tangle-grid.c
 *
 * This file is part of Tangle Toolkit - A graphical actor library based on Clutter Toolkit
 *
 * (c) 2010 Henrik Hedberg <henrik.hedberg@innologies.fi>
 *
 */

#include "tangle-grid.h"

/**
 * SECTION:tangle-grid
 * @Short_description: An actor that displays grid lines
 * @Title: TangleGrid
 */
 
G_DEFINE_TYPE(TangleGrid, tangle_grid, TANGLE_TYPE_ACTOR);

enum {
	PROP_0,
	PROP_GRID_SPACING_X,
	PROP_GRID_SPACING_Y,
	PROP_BACKGROUND_COLOR,
	PROP_LINE_COLOR
};

struct _TangleGridPrivate {
	gfloat grid_spacing_x;
	gfloat grid_spacing_y;
	ClutterColor background_color;
	ClutterColor line_color;
	guint background_color_set : 1;
	guint line_color_set : 1;
};

static ClutterColor black = { 0, 0, 0, 255 };

ClutterActor* tangle_grid_new(gfloat grid_spacing_x, gfloat grid_spacing_y) {

	return CLUTTER_ACTOR(g_object_new(TANGLE_TYPE_GRID, "grid-spacing-x", grid_spacing_x, "grid-spacing-y", grid_spacing_y, NULL));
}

ClutterActor* tangle_grid_new_with_line_color(gfloat grid_spacing_x, gfloat grid_spacing_y, ClutterColor* line_color) {

	return CLUTTER_ACTOR(g_object_new(TANGLE_TYPE_GRID, "grid-spacing-x", grid_spacing_x, "grid-spacing-y", grid_spacing_y, "line-color", line_color, NULL));
}

gfloat tangle_grid_get_grid_spacing_x(TangleGrid* grid) {

	return grid->priv->grid_spacing_x;
}

void tangle_grid_set_grid_spacing_x(TangleGrid* grid, gfloat grid_spacing_x) {
	g_return_if_fail(grid_spacing_x >= 0.0);

	if (grid->priv->grid_spacing_x != grid_spacing_x) {
		grid->priv->grid_spacing_x = grid_spacing_x;
		clutter_actor_queue_redraw(CLUTTER_ACTOR(grid));
		g_object_notify(G_OBJECT(grid), "grid-spacing-x");
	}
}

gfloat tangle_grid_get_grid_spacing_y(TangleGrid* grid) {

	return grid->priv->grid_spacing_y;
}

void tangle_grid_set_grid_spacing_y(TangleGrid* grid, gfloat grid_spacing_y) {
	g_return_if_fail(grid_spacing_y >= 0.0);

	if (grid->priv->grid_spacing_y != grid_spacing_y) {
		grid->priv->grid_spacing_y = grid_spacing_y;
		clutter_actor_queue_redraw(CLUTTER_ACTOR(grid));
		g_object_notify(G_OBJECT(grid), "grid-spacing-y");
	}
}

void tangle_grid_get_background_color(TangleGrid* grid, ClutterColor* color) {
	g_return_if_fail(color != NULL);

	*color = grid->priv->background_color;
}

void tangle_grid_set_background_color(TangleGrid* grid, const ClutterColor* color) {
	if (!clutter_color_equal(&grid->priv->background_color, color)) {
		if (color) {
			grid->priv->background_color = *color;
			grid->priv->background_color_set = TRUE;
		} else {
			grid->priv->background_color_set = FALSE;
		}
		g_object_notify(G_OBJECT(grid), "background-color");
		clutter_actor_queue_redraw(CLUTTER_ACTOR(grid));
	}
}

void tangle_grid_get_line_color(TangleGrid* grid, ClutterColor* color) {
	g_return_if_fail(color != NULL);

	*color = grid->priv->line_color;
}

void tangle_grid_set_line_color(TangleGrid* grid, const ClutterColor* color) {
	if (!clutter_color_equal(&grid->priv->line_color, color)) {
		if (color) {
			grid->priv->line_color = *color;
			grid->priv->line_color_set = TRUE;
		} else {
			grid->priv->line_color_set = FALSE;
		}
		g_object_notify(G_OBJECT(grid), "line-color");
		clutter_actor_queue_redraw(CLUTTER_ACTOR(grid));
	}
}

static void tangle_grid_paint_aligned(TangleActor* actor, gfloat width, gfloat height) {
	TangleGrid* grid;
	gfloat x, y;
	guint8 alpha;
	
	grid = TANGLE_GRID(actor);
	
	if (grid->priv->background_color_set) {
		alpha = clutter_actor_get_paint_opacity(CLUTTER_ACTOR(actor)) * grid->priv->background_color.alpha / 255;
		cogl_set_source_color4ub(grid->priv->background_color.red,
			 		 grid->priv->background_color.green,
			 		 grid->priv->background_color.blue,
					 alpha);
		cogl_rectangle(0.0, 0.0, width, height);
	}

	if (grid->priv->line_color_set) {
		alpha = clutter_actor_get_paint_opacity(CLUTTER_ACTOR(actor)) * grid->priv->line_color.alpha / 255;
		cogl_set_source_color4ub(grid->priv->line_color.red,
			 	 grid->priv->line_color.green,
			 	 grid->priv->line_color.blue,
				 alpha);

		if (grid->priv->grid_spacing_x > 0.0) {
			for (x = 0.0; x < width; x += grid->priv->grid_spacing_x) {
				cogl_path_move_to(x, 0.0);
				cogl_path_line_to(x, height);
			}
			cogl_path_stroke();
		}
		if (grid->priv->grid_spacing_y > 0.0) {
			for (y = 0.0; y < height; y += grid->priv->grid_spacing_y) {
				cogl_path_move_to(0.0, y);
				cogl_path_line_to(width, y);
			}
			cogl_path_stroke();
		}
	}
}

static void tangle_grid_set_property(GObject* object, guint prop_id, const GValue* value, GParamSpec* pspec) {
	TangleGrid* grid;
	
	grid = TANGLE_GRID(object);

	switch (prop_id) {
		case PROP_GRID_SPACING_X:
			tangle_grid_set_grid_spacing_x(grid, g_value_get_float(value));
			break;
		case PROP_GRID_SPACING_Y:
			tangle_grid_set_grid_spacing_y(grid, g_value_get_float(value));
			break;
		case PROP_BACKGROUND_COLOR:
			tangle_grid_set_background_color(grid, clutter_value_get_color(value));
			break;
		case PROP_LINE_COLOR:
			tangle_grid_set_line_color(grid, clutter_value_get_color(value));
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
			break;
	}
}

static void tangle_grid_get_property(GObject* object, guint prop_id, GValue* value, GParamSpec* pspec) {
        TangleGrid* grid;

	grid = TANGLE_GRID(object);

        switch (prop_id) {
		case PROP_GRID_SPACING_X:
			g_value_set_float(value, grid->priv->grid_spacing_x);
			break;
		case PROP_GRID_SPACING_Y:
			g_value_set_float(value, grid->priv->grid_spacing_y);
			break;
		case PROP_BACKGROUND_COLOR:
			clutter_value_set_color(value, &grid->priv->background_color);
			break;
		case PROP_LINE_COLOR:
			clutter_value_set_color(value, &grid->priv->line_color);
			break;
	        default:
		        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
		        break;
        }
}

static void tangle_grid_finalize(GObject* object) {
	G_OBJECT_CLASS(tangle_grid_parent_class)->finalize(object);
}

static void tangle_grid_dispose(GObject* object) {
	G_OBJECT_CLASS(tangle_grid_parent_class)->dispose(object);
}

static void tangle_grid_class_init(TangleGridClass* grid_class) {
	GObjectClass* gobject_class = G_OBJECT_CLASS(grid_class);
	TangleActorClass* actor_class = TANGLE_ACTOR_CLASS(grid_class);

	gobject_class->finalize = tangle_grid_finalize;
	gobject_class->dispose = tangle_grid_dispose;
	gobject_class->set_property = tangle_grid_set_property;
	gobject_class->get_property = tangle_grid_get_property;

	actor_class->paint_aligned = tangle_grid_paint_aligned;

	/**
	 * TangleGrid:grid-spacing-x:
	 */
	g_object_class_install_property(gobject_class, PROP_GRID_SPACING_X,
	                                g_param_spec_float("grid-spacing-x",
	                                                   "Grid spacing x",
	                                                   "The horizontal spacing of grid lines",
	                                                   0.0, G_MAXFLOAT, 0.0,
	                                                   G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
	/**
	 * TangleGrid:grid-spacing-y:
	 */
	g_object_class_install_property(gobject_class, PROP_GRID_SPACING_Y,
	                                g_param_spec_float("grid-spacing-y",
	                                                   "Grid spacing y",
	                                                   "The vertical spacing of grid lines",
	                                                   0.0, G_MAXFLOAT, 0.0,
	                                                   G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
	/**
	 * TangleGrid:background-color:
	 */
	g_object_class_install_property(gobject_class, PROP_BACKGROUND_COLOR,
	                                clutter_param_spec_color("background-color",
	                                                	 "Background color",
	                                                	 "The background color of the grid",
	                                                	 NULL,
	                                                	 G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
	/**
	 * TangleGrid:line-color:
	 */
	g_object_class_install_property(gobject_class, PROP_LINE_COLOR,
	                                clutter_param_spec_color("line-color",
	                                                	 "Line color",
	                                                	 "The line stroke color of the grid",
	                                                	 &black,
	                                                	 G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));


	g_type_class_add_private (gobject_class, sizeof (TangleGridPrivate));
}

static void tangle_grid_init(TangleGrid* grid) {
	grid->priv = G_TYPE_INSTANCE_GET_PRIVATE(grid, TANGLE_TYPE_GRID, TangleGridPrivate);
}
